/****************************************************************************
				Program INtegrity CHecker
				for NetWare 3.x
				Copyright 1992, Novell, Inc.

				Morgan B. Adair
				Novell Systems Research

				DATAFILE.C		Functions to manage Btrieve database
*****************************************************************************/

#include <stdio.h>
#include <sys/stat.h>
#include "datafile.h"
#include "btrieve.h"
#include "recdecl.h"
#include "log.h"

char fileName[] = "SYS:\\SYSTEM\\PINCH.BTR";			/* Btrieve file name */
char positionBlock[128];									/* Btrieve file position block */

short DataFileExists(void)
{
	struct stat	statblk;
	int	ccode;

	/* assume that if file is there, it's the right file */

	ccode = stat(fileName, &statblk);
	return(!ccode);
}

short OpenFile(void)
{
	unsigned short	bufferLength;
	FILE_ENTRY		entry;
	short 			status;

	/* Figure out how long the record is */
   bufferLength = sizeof(FILE_ENTRY);

	/* Call BTRIEVE Open operation */
   status = btrv(BT_OPEN, positionBlock, (char *)&entry, &bufferLength, fileName, 1);

	/* Return the result */
   return(status);
}

short CreateDataFile(void)
{
	unsigned short	bufferLength;
	FILE_SPEC		fileBuffer;
	short 			status;

	/* Create a new Btrieve file for the program file data */
	/* Initialize buffer */
	memset(&fileBuffer, 0, sizeof(fileBuffer));

	/* Define the global file paramaters */
	fileBuffer.recordLength = sizeof(FILE_ENTRY);
	fileBuffer.pageSize     = 4096;
	fileBuffer.fileFlags    = 0;
	fileBuffer.indexCount   = NUMBER_OF_KEYS;
	fileBuffer.preAlloc     = 0;

	/* Key 0: File Path */
	fileBuffer.keyBuf[0].keyPos  = 1;
	fileBuffer.keyBuf[0].keyLen  = KEY0_LEN;
	fileBuffer.keyBuf[0].keyFlag = BT_KEY_EXT_TYPE;
	fileBuffer.keyBuf[0].keyType = BT_KEY_ZSTR_TYPE;

	/* Figure out the data buffer length */
	bufferLength = sizeof(fileBuffer);

	/*--- Call Btrieve with a Create request. ---*/
	status = btrv(BT_CREATE, positionBlock, (char *)&fileBuffer, &bufferLength, fileName, 0);

	if (status != 0) {
		printf("File creation error %d.\n",status);
		PressAnyKeyToContinue();
	}

	return(status);
}

short DeleteItem(char *Name)
{
	short				status;
	unsigned short	dataLength;
	char				keyBuffer[KEY0_LEN];
	FILE_ENTRY		entry1;
	FILE_ENTRY		entry2;

	/* Find the first record with path matching the specification */
	dataLength = sizeof(FILE_ENTRY);
	strcpy(keyBuffer, Name);
	status = btrv(BT_GET_EQUAL, positionBlock, (char *)&entry1, &dataLength, keyBuffer,0);

	if (status != BT_SUCCESS)
		return(status);

	/*--- Now, delete it ---*/
	status = btrv(BT_DELETE, positionBlock, (char *)&entry1, &dataLength, keyBuffer, 0);

	return(status);
}

short InsertItem(FILE_ENTRY *entry)
{
	short				status;
	unsigned short	dataLength;
	char				keyBuffer[KEY0_LEN];

	/* Figure out the data buffer length */
	dataLength = sizeof(FILE_ENTRY);

	/* Call Btrieve with Add request */
	status = btrv(BT_INSERT, positionBlock, (char *)entry, &dataLength, keyBuffer, 0);
	return(status);
}

short SearchItem(int keyNum, unsigned short dataLength, char *data, FILE_ENTRY *entry)
/* Search for all items that match a simple specification.
	The specification is a key number and an exact-match value. */
{
	short				status;
	char				keyBuffer[KEY0_LEN];
	unsigned short	dataBufferLength;

	/* Initialize keyBuffer */
	memset(keyBuffer, 0, KEY0_LEN);

	/* Copy the key data to somewhere that can be overwritten */
  	memcpy(keyBuffer, data, dataLength);

	dataBufferLength = sizeof(FILE_ENTRY);

	/* Find the first item matching the specification */
	status = btrv(BT_GET_EQUAL, positionBlock, (char *)entry, &dataBufferLength, keyBuffer, keyNum);

	/* Die on errors other than ERR_END_OF_FILE */
	if (status != BT_SUCCESS)
		return(status);

	return(BT_SUCCESS);
}

int UpdateItem(FILE_ENTRY *entry)
{
	int	status;
	int	dataLength;
	char	keyBuffer[KEY0_LEN];

	/* Figure out the data buffer length */
	dataLength = sizeof(FILE_ENTRY);

	/* Call Btrieve with Update request */
	status = btrv(BT_UPDATE, positionBlock, (char *)entry, &dataLength, keyBuffer, 0);
	return(status);
}

short DeleteRecordsForDeletedFiles(void)
{
	short				status;
	unsigned short	dataLength;
	char				keyBuffer[KEY0_LEN];
	FILE_ENTRY		entry;
	char				textBuffer[KEY0_LEN+20];
	int				i;

	/* Figure out the data buffer length */
	dataLength = sizeof(FILE_ENTRY);

	/* Get the first item in the database */
	status = btrv(BT_GET_FIRST, positionBlock, (char *)&entry, (int *)&dataLength, keyBuffer, 0);

	i=1;
	/* As long as there are more, keep going */
	while (status != BT_END_OF_FILE) {

		/* Die on errors other than ERR_END_OF_FILE */
		if (status != BT_SUCCESS)
			return(status);

		i = i+1;

		/* See if file was found on last scan of file system */
		if (entry.checked) {
			entry.checked = FALSE;
			status = UpdateItem(&entry);
			if (status != BT_SUCCESS) {
				sprintf(textBuffer, "Unable to update record %s, result = %d\n", entry.pathName, status);
				LogMessage(textBuffer);
			}
		} else {
			sprintf(textBuffer, "File %s was deleted\n", entry.pathName);
			LogMessage(textBuffer);
			status = btrv(BT_DELETE, positionBlock, (char *)&entry, &dataLength, keyBuffer, 0);
		}

		/* Step forward to the next item */
		status = BTRV(BT_GET_NEXT, positionBlock, (char *)&entry, (int *)&dataLength, keyBuffer, 0);
	}

	return(BT_SUCCESS);
}

short CloseDataFile(void)
{
	unsigned short	bufferLength;
	FILE_ENTRY		entry;
	short				status;

	/* Call Btrieve with Close operation */
	status = btrv(BT_CLOSE, positionBlock, (char *)&entry, &bufferLength, fileName, 0);

	return(status);
}

